<?php

namespace Drupal\campaignmonitor\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\campaignmonitor\CampaignMonitorManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a confirmation form for deleting mymodule data.
 */
class CampaignMonitorListDeleteForm extends ConfirmFormBase {

  /**
   * @var Drupal\campaignmonitor\CampaignMonitorManager $campaignmonitor_manager
   */
  protected $campaignMonitorManager;

  /**
   * The ID of the item to delete.
   *
   * @var string
   */
  protected $id;

  /**
   * Class constructor.
   */
  public function __construct(
    CampaignMonitorManager $campaignmonitor_manager
  ) {

    $this->campaignMonitorManager = $campaignmonitor_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    // Instantiates this form class.
    return new static(
      // Load the service required to construct this class.
      $container->get('campaignmonitor.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'campaignmonitor_list_delete';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $options = $this->campaignMonitorManager->getExtendedList($this->id);
    return t('Do you want to delete %name?', ['%name' => $options['name']]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('campaignmonitor.lists');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return t('Only do this if you are sure!');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelText() {
    return t('Cancel');
  }

  /**
   * {@inheritdoc}
   *
   * @param int $list_id
   *   (optional) The ID of the item to be deleted.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $list_id = NULL) {
    $this->id = $list_id;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($this->campaignMonitorManager->deleteList($this->id())) {
      drupal_set_message(t('The list has been deleted.'), 'status');
    }
    else {
      drupal_set_message(t('The list could not be deleted.'), 'error');
    }
    $url = Url::fromRoute('campaignmonitor.refresh_lists');
    $form_state->setRedirectUrl($url);
  }

}
